/*
 * ContactsViewer.java
 *
 *  Lovely Products 2007 (richard@movethemarkets.com)
 * GPL v2 or later.
 */

package com.lovely.bin;

import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.system.*;
import java.util.*;
import org.json.me.*;

// *********************************************************
// *********************************************************
// *********************************************************
// *********************************************************
class ContactsReloadMenuItem extends MenuItem {
   private String url;
   public ContactsReloadMenuItem(String url_) {
      super("Reload",0,0);
      url = url_;
   }    
   
   public void run() {
      ContactsViewer jv = new ContactsViewer(url);
      UiApplication uiapp = UiApplication.getUiApplication(); 
      uiapp.popScreen(uiapp.getActiveScreen());  // get rid of this screen
      UiApplication.getUiApplication().pushScreen(jv);
      jv.pullData();
   }
}

// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************

class ContactTextField extends RichTextField {
   private String author;
   private String theURL;
   public ContactTextField(String author_,
                          String theURL_,
                          String txt,
                          int[] offsets, 
                          byte[] attributes,
                          Font[] fonts, 
                          long style) {
      super(txt,offsets,attributes,fonts,style);
      author = author_;    
      theURL = sanitizeURL(theURL_);
   }    
   
   private String sanitizeURL(String url) {
       int indx = url.lastIndexOf('#');
       if(indx != -1) {
          url = url.substring(0,indx);
       }
       return url; 
   }
   
   public String getAuthor() { return author; }
   public String getURL() { return theURL; }   
}

// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************
class ContactDataReader extends Thread implements com.lovely.bin.IStatusable {
   private ContactsViewer cv;
   private String title;
   private Vector fields;
   public ContactDataReader(ContactsViewer cv_) {
       cv = cv_;
       title = "";
       fields = new Vector();
   }
         
   public void doAlert(final String alstr) {
      UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                Dialog.alert(alstr);
            }
        });   
   }
      
    private RichTextField parseContact(JSONObject contact) throws JSONException {
       StringBuffer answer = new StringBuffer();
       String theAuthor =  contact.getString("nick");
       String theURL = contact.getString("url");
       
       answer.append(theAuthor);
       answer.append(": ");
       answer.append(contact.getString("first_name"));
       answer.append(" ");
       answer.append(contact.getString("last_name"));

       int[] offsets = new int[3];
       offsets[0] = 0;
       offsets[1] = theAuthor.length();
       offsets[2] = answer.length();
       Font[] fonts = new Font[2];
       fonts[1] = Font.getDefault();
       fonts[0] = fonts[1].derive(Font.BOLD);
       byte[] attributes = new byte[2];
       attributes[0] = 0;
       attributes[1] = 1;
       RichTextField rtf = new ContactTextField(theAuthor,
                                                theURL,
                                                answer.toString(),
                                                offsets,
                                                attributes,
                                                fonts,
                                                Field.READONLY); 
       return rtf;
    }
       
    
    public void setStatus(final String status) {
      UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                cv.setStatus(status);
            }
        });   
    }

    private void setFields() {
      UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                cv.setFields(title,fields);
            }
        });   
    }
    
   public void run() {
        String jsonString = HTTPInterface.readURL(cv.getURL(),this);
        setStatus("Done reading... formatting page...");

        try {
           JSONObject mainObj = new JSONObject(jsonString);
           if(mainObj.has("contacts")) {
             JSONArray contacts = mainObj.getJSONArray("contacts");
             title = "Contacts for " + mainObj.getString("nick");
             for(int i = 0; i < contacts.length(); ++i) {
                  JSONObject contact = contacts.getJSONObject(i);
                  RichTextField item = parseContact(contact);             
                  fields.addElement(item);
             }
          } else {
             title = "Contacts unavailable.";
             fields.addElement(new RichTextField("This user's contacts are private."));    
          }
        } catch(Exception e) {
          title = "ERROR!";
          fields.removeAllElements();
          fields.addElement(new RichTextField(e.toString())); 
        }
        setFields();
   }       
}

// ****************************************************************
// ****************************************************************
// ****************************************************************

// used to change the menu as various authors gain the focus... used
// to drill down to other user's jaikus.
class ContactsFocusMenuAdjuster implements FocusChangeListener {
   private GetUserJaikuMenuItem userCommand;
   private GetUserContactsMenuItem contactsCommand;
   
   public ContactsFocusMenuAdjuster(GetUserJaikuMenuItem userCommand_,
                                    GetUserContactsMenuItem contactsCommand_) {
      userCommand = userCommand_;
      contactsCommand = contactsCommand_;
   }
   
   public void focusChanged(Field f, int typ) {
     if(typ == FocusChangeListener.FOCUS_GAINED) {
        ContactTextField ctf = (ContactTextField)f;
        String author = ctf.getAuthor();
        userCommand.setUserName(author);
        if(author.charAt(0) != '#') contactsCommand.setUserName(author);
        else contactsCommand.setUserName(null);
     }    
   }
}


// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************

/**
 *   Creates a screen from a JSON feed of jaiku content.
 */
public class ContactsViewer extends MainScreen {
    private String url;
    private GetUserJaikuMenuItem getUserJaikuMenuItem; // used when drilling down...
    private GetUserContactsMenuItem getUserContactsMenuItem; // used for drilling down...
    private LabelField statusField;
    
    public ContactsViewer(String url_) {
        super();
        url = url_;
        getUserJaikuMenuItem = new GetUserJaikuMenuItem();
        getUserContactsMenuItem = new GetUserContactsMenuItem();
        
        LabelField title = new LabelField
         ("JaikuBerry Loading Page...", LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(title);
        statusField = new LabelField("Making Connection...");
        add(statusField);
    }
    
    public String getURL() { return url; }
    
    public void pullData() {
       // spawn a thread to read in the URL...
       ContactDataReader cdr = new ContactDataReader(this);
       cdr.start();    
    }
    
    // make our menu...
    public void makeMenu(Menu m, int i) {
       m.add(new ContactsReloadMenuItem(url));
       m.add(getUserJaikuMenuItem);   
       m.add(getUserContactsMenuItem); 
       super.makeMenu(m,i);
    }   
    
    // expected to be called from the event thread
    public void setStatus(String status) {
        statusField.setText(status);
    }
    
    // expected to be called from the event thread...
    public void setFields(String title, Vector fieldV) {
       setTitle(new LabelField(title));
       
       // first remove the status field...
       delete(statusField);
       
       // if we're allowing drilldown, set the focus listener so we
       // always have the right contextual menu in place..
       FocusChangeListener ffl = new ContactsFocusMenuAdjuster(getUserJaikuMenuItem,getUserContactsMenuItem);           
      
       
       // now add in all the fields we have...
       for (Enumeration e = fieldV.elements() ; e.hasMoreElements() ;) {
          Field fld = (Field)e.nextElement();
          if(fld instanceof ContactTextField) {
              fld.setFocusListener(ffl);
          }
          add(fld);
          add(new SeparatorField());
       }
    }
     
    protected boolean keyCharUnhandled(char key, int status, int time) {
      boolean retval = false;
      
      if(key == Characters.LATIN_SMALL_LETTER_T) {
         scroll(Manager.TOPMOST);    
      } else if(key == Characters.LATIN_SMALL_LETTER_B) {
         scroll(Manager.BOTTOMMOST);              
      } else if(key == Characters.LATIN_SMALL_LETTER_R) {
         new ContactsReloadMenuItem(this.url).run();    
      } else if((key == Characters.LATIN_SMALL_LETTER_J) || 
                (key == Characters.ENTER)) {
         Field f = getFieldWithFocus();
         if(f instanceof ContactTextField) {
           ContactTextField ctf = (ContactTextField)f;
           if(ctf.getAuthor() != null) {
             JaikuViewer jv = JaikuViewer.usersJaikusForm(ctf.getAuthor());
             UiApplication.getUiApplication().pushScreen(jv);
             jv.pullData();  
           }
         }    
      }
      else retval = super.keyCharUnhandled(key,status,time);
  
      return retval;
    }

    public static ContactsViewer contactsForm(String user) {
        StringBuffer url = new StringBuffer("http://");
        url.append(user).append(".jaiku.com/json");
        return new ContactsViewer(url.toString());   
    }
   
}
