/*
 * JaikuAppSettings.java
 *  Lovely Products 2007 (richard@movethemarkets.com)
 * GPL v2 or later.
 */

package com.lovely.bin;

 /**
 * This class is in charge of retrieving and storing the user's settings...
 */
public class JaikuAppSettings {
  private AppSettings appSettings;
  private String user_name;
  private String personal_key;
  private int default_location; // the index into the locations...
  private String[] location;
  private boolean settingsChanged_p;
  
  public JaikuAppSettings() {
     appSettings = new AppSettings("jaikuAppSettings");
     settingsChanged_p = false;
     location = new String[JaikuMIDletConstants.NUM_LOCATIONS];
     String unset_string = "unset";
     
     user_name = appSettings.getSetting("UserName");
     if(user_name == null) {
        user_name = unset_string;   
     }
     location[0] = appSettings.getSetting("Location");
     if(location[0] == null) {
        location[0] = unset_string;  
     }
     for(int i = 1; i < location.length; ++i) {
       location[i] = appSettings.getSetting("Location" + Integer.toString(i));
       if(location[i] == null) {
          location[i] = unset_string;    
       } 
     }
     personal_key = appSettings.getSetting("PersonalKey");
     if(personal_key == null) {
        personal_key = unset_string;
     }
     
     String defloc = appSettings.getSetting("DefaultLoc");
     if(defloc == null) {
       default_location = 0;    
     } else {
       default_location = Integer.parseInt(defloc);    
     }
     
     // be careful if we ever shrink the number of allowed locations...
     if((default_location < 0) || (default_location >= JaikuMIDletConstants.NUM_LOCATIONS)) 
       default_location = 0;
  }
 
  public String getUserName() { return user_name; }
  public String[] getLocation() { return location; }
  public String getPersonalKey() { return personal_key; }
  public int getDefaultLocationIndex() { return default_location; }
  
  public void updateSettings(String un, 
                             String pkey, 
                             String[] locs) {
      // if any have changed, notice that...
      if((!un.equals(user_name)) ||
         (!pkey.equals(personal_key))) {
          settingsChanged_p = true;     
      } 
      for(int i = 0; i < location.length; ++i) {
         if(!locs[i].equals(location[i])) 
             settingsChanged_p = true;
      }
      
      user_name = un;
      personal_key = pkey;
      for(int i = 0; i < location.length; ++i)
        location[i] = locs[i];
  }
 
  /**
   * determines whether our application has had settings set yet
   * @return true if we are still at default settings.
   */
  public boolean atDefaults() {
     return ((user_name.equals("unset")) || (personal_key.equals("unset")));    
  }
  
  public void save(int whichLocation) {
     if(whichLocation != default_location) {
        appSettings.setSetting("DefaultLoc",Integer.toString(whichLocation));    
     }
     if(settingsChanged_p) {
        appSettings.setSetting("UserName",user_name);
        appSettings.setSetting("Location",location[0]);
        for(int i = 1; i < location.length; ++i)          
          appSettings.setSetting("Location" + Integer.toString(i),location[i]);
        appSettings.setSetting("PersonalKey",personal_key);
     }   
     settingsChanged_p = false;
     appSettings.close(); 
  }
}

 
