/*
 * JaikuBerry.java
 *
 *  Lovely Products 2007 (richard@movethemarkets.com)
 * GPL v2 or later.
 */

package com.lovely.bin;

import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.system.*;

public class JaikuBerry extends UiApplication {
    public static void main(String argv[]) {
        JaikuBerry app = new JaikuBerry();
        app.enterEventDispatcher();
    }

    public JaikuBerry() {
        pushScreen(new PostScreen());
    }
}

class CharacterCounter implements FieldChangeListener {
    private RichTextField statusField;
    public CharacterCounter(RichTextField rtf) {
      statusField = rtf;
    }
    
    public void   fieldChanged(Field field, int context)  {
         statusField.setText(Integer.toString(140 - ((BasicEditField)field).getTextLength()));
    }  
}

class SettingsMenuItem extends MenuItem {
   private JaikuAppSettings jas;
   public SettingsMenuItem(JaikuAppSettings jas_) {
      super("Settings",2,2);    
      jas = jas_;
   }    
   public void run() {
      UiApplication.getUiApplication().pushScreen(new JaikuBerrySettings(jas));
   }
}
class AboutMenuItem extends MenuItem {
   public AboutMenuItem() {
      super("About",3,3);   
   }    
   public void run() {
      UiApplication.getUiApplication().pushScreen(new JaikuBerryAbout());
   }
}

class MyContactsMenuItem extends MenuItem {
   private JaikuAppSettings jas;
   public MyContactsMenuItem(JaikuAppSettings jas_) {
      super("My Contacts",2,2);    
      jas = jas_;
   }   
   public void run() {
      if(jas.atDefaults()) {
         Dialog.alert(JaikuMIDletConstants.getUnsetError());
         return;    
      }
      ContactsViewer cv = ContactsViewer.contactsForm(jas.getUserName());
      UiApplication.getUiApplication().pushScreen(cv); 
      cv.pullData(); 
   }
}

class ViewPostsMenuItem extends MenuItem {
   private JaikuAppSettings jas;
   public ViewPostsMenuItem(JaikuAppSettings jas_) {
      super("My Posts",1,1);    
      jas = jas_;
   }   
   public void run() {
      if(jas.atDefaults()) {
         Dialog.alert(JaikuMIDletConstants.getUnsetError());
         return;    
      }
      JaikuViewer jv = JaikuViewer.usersJaikusForm(jas.getUserName());
      UiApplication.getUiApplication().pushScreen(jv); 
      jv.pullData();   
   }
}

class ViewOverviewMenuItem extends MenuItem {
    private JaikuAppSettings jas;
   public ViewOverviewMenuItem(JaikuAppSettings jas_) {
      super("Overview",1,1);    
      jas = jas_;
   }   
   public void run() {
      if(jas.atDefaults()) {
         Dialog.alert(JaikuMIDletConstants.getUnsetError());
         return;    
      }
      JaikuViewer jv = JaikuViewer.usersOverviewForm(jas.getUserName(),jas.getPersonalKey());
      UiApplication.getUiApplication().pushScreen(jv); 
      jv.pullData();   
   }
}

class ExploreMenuItem extends MenuItem {
   public ExploreMenuItem() {
      super("Explore",1,1);    
   }   
   public void run() {
      JaikuViewer jv = JaikuViewer.exploreForm();
      UiApplication.getUiApplication().pushScreen(jv); 
      jv.pullData();   
   }
}

class SendPostMenuItem extends MenuItem {
   PostScreen ps;
   JaikuAppSettings jas;
   public SendPostMenuItem(PostScreen ps_,JaikuAppSettings jas_) {
      super("Post",0,0);
      ps = ps_;
      jas = jas_;
   }    
   
   public void run() {
      if(jas.atDefaults()) {
         Dialog.alert(JaikuMIDletConstants.getUnsetError());
         return;    
      }
      ps.sendPost();
   }
}

class PostScreen extends MainScreen {
    AutoTextEditField postField;
    RichTextField statusLine;
    JaikuAppSettings jas;
    ObjectChoiceField pictureSelection,locationSelection;
    PostSender postSender;
    
    public PostScreen() {
        super();
        postSender = null;
        jas = new JaikuAppSettings();
        LabelField title = new LabelField
         ("JaikuBerry", LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(title);
        postField = new AutoTextEditField("Post: ","",140,Field.EDITABLE|BasicEditField.NO_NEWLINE);
        add(postField);
        pictureSelection = new ObjectChoiceField("Icon: ",PictureChoices.pictures,0,Field.EDITABLE);
        locationSelection = new ObjectChoiceField("Location: ",
                                                  jas.getLocation(),
                                                  jas.getDefaultLocationIndex(),
                                                  Field.EDITABLE);
        add(pictureSelection);
        add(locationSelection);
        add(new SeparatorField());
        add(new LabelField("Status:"));
        statusLine = new RichTextField("ok",Field.NON_FOCUSABLE|Field.READONLY);
        statusLine.setEditable(false);
        add(statusLine);
        postField.setChangeListener(new CharacterCounter(statusLine));
    }
 
    protected boolean keyCharUnhandled(char key, int status, int time) {
      boolean retval = false;
      
      if(key == Characters.ENTER) {
        if(jas.atDefaults()) {
           Dialog.alert(JaikuMIDletConstants.getUnsetError());
        } else {
           sendPost();
        }
      } else if(key == Characters.ESCAPE) {
         onClose();
         retval = true;    
      }
      
      return retval;  // not closing the screen   
    }

    protected void makeMenu(Menu m, int instance) {
        m.add(new SendPostMenuItem(this,jas));
        m.add(new ViewOverviewMenuItem(jas));
        m.add(new ViewPostsMenuItem(jas));
        m.add(new ExploreMenuItem());
        m.add(new MyContactsMenuItem(jas));
        m.addSeparator();
        m.add(new SettingsMenuItem(jas));
        m.add(new AboutMenuItem());
        super.makeMenu(m,instance);
    }
    
    
    // when we are re-exposed, re-set the location choices, since they
    // may have changed...
    protected void onExposed() {
       super.onExposed();
       int selected = locationSelection.getSelectedIndex();
       locationSelection.setChoices(jas.getLocation());   
       locationSelection.setSelectedIndex(selected);
    }
    
    // meant to be called from the event thread...
    public void sendPost() {
       if(!postField.isEditable()) {
           Dialog.alert("Already Sending A Post!");
           return;  
       }
       postSender = new PostSender(this);
       postSender.postInfo(jas.getUserName(),
                           jas.getPersonalKey(),
                           (String)locationSelection.getChoice(locationSelection.getSelectedIndex()),
                           postField.getText(),
                           PictureChoices.numbers[pictureSelection.getSelectedIndex()]);
       postField.setEditable(false);
       statusLine.setText("Sending... (please wait)");         
       postSender.start();  // start the thread!
    }
    
    public void postSent() {
       UiApplication.getUiApplication().invokeLater(new Runnable()
       {
          public void run()
          {
            try  {  postSender.join(); } catch(Exception e) { }
            
            if(postSender.success()) {
              postField.setText("");
              pictureSelection.setSelectedIndex(0);
              statusLine.setText("Post sent successfully."); 
            }
            else {
              statusLine.setText("Error Sending Post!");
            }
            postSender = null;
            postField.setEditable(true);          
          }
       });
    }
    
    public boolean onClose() {
       jas.save(locationSelection.getSelectedIndex());
       System.exit(0);
       return true;
    }
}
