/*
 * JaikuMIDlet.java
 *
 *  Lovely Products 2007 (richard@movethemarkets.com)
 * GPL v2 or later.
 */

package com.lovely.bin;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;

/**
 *  this is the screen for managing settings.
 */
class SettingsForm extends Form implements CommandListener {
    private Screen back;
    private Display display;
    private JaikuAppSettings settings;
    
    // the fields for settings to fill in
    private TextField uname_field;
    private TextField pkey_field;
    private TextField loc_field;
    
    public SettingsForm(JaikuAppSettings sd, Screen back_, Display d) {
      super("Jaiku Posting MIDlet Settings");
      settings = sd;
      back = back_;
      display = d;
      
      // fields
      uname_field = new TextField("User Name:",sd.getUserName(),80,TextField.ANY);
      pkey_field = new TextField("Personal Key:",sd.getPersonalKey(),80,TextField.ANY);
      loc_field = new TextField("Location:",sd.getLocation(),80,TextField.ANY);
      append(uname_field);
      append(pkey_field);
      append(loc_field); 
      
      // commands
      addCommand(new Command("Back",Command.BACK,0));
      addCommand(new Command("Save",Command.OK,1)); 
      setCommandListener(this); 
    }    
    
    public void commandAction(Command c, Displayable s) {
         if(c.getLabel().equals("Save")) {
            settings.updateSettings(uname_field.getString(),
                                    pkey_field.getString(),
                                    loc_field.getString());   
         }
         
         // whether they saved or not, we now go back to the previous screen...
         display.setCurrent(back);
    }
}

/**
 * The info screen... just some text.
 */
class JaikuInfo extends Form implements CommandListener {
     private Screen back;
     private Display display;
     
     public JaikuInfo(Screen back_, Display d) {
         super("About the Jaiku Posting MIDlet");
         back = back_;
         display = d;
         append(new StringItem("Version:",JaikuMIDletConstants.getVersion()));
         append(new StringItem("Author:","Richard Todd"));
         append(new StringItem("Email:","richard@movethemarkets.com"));
         addCommand(new Command("Back",Command.BACK,0));
         setCommandListener(this);
     }
     
    public void commandAction(Command c, Displayable s) {
         display.setCurrent(back);
    }
}

public class JaikuMIDlet
    extends MIDlet 
    implements CommandListener, ItemStateListener {
  private Display display; // the main display manager
  
  // the main form...  
  private Form mainForm;
  private TextField post;
  private StringItem status_string;
  private ChoiceGroup post_icon;
  
  // Stored Persistent Data
  private JaikuAppSettings settings;
  
  // Our ability to send posts...
  private PostSender psend;
  
  public JaikuMIDlet() {
    display = Display.getDisplay(this);
    psend = null;
    
    mainForm = new Form("Jaiku Posting MIDlet");
    // commands...
    mainForm.addCommand(new Command("Post",Command.OK,0));
    mainForm.addCommand(new Command("View Overview",Command.SCREEN,1));
    mainForm.addCommand(new Command("View User",Command.SCREEN,1));
    mainForm.addCommand(new Command("Settings",Command.SCREEN,2));
    mainForm.addCommand(new Command("Exit", Command.EXIT, 1));
    mainForm.addCommand(new Command("About",Command.SCREEN,2));
    // ui elements
    post = new TextField("Post:","",140,TextField.ANY);
    mainForm.append(post);
    post_icon = new ChoiceGroup("Icon:",Choice.POPUP,PictureChoices.pictures,null);
    mainForm.append(post_icon);
    status_string = new StringItem("Status:","ok");
    mainForm.append(status_string);
             
    // the personal data 
    settings = new JaikuAppSettings();    
    mainForm.setCommandListener(this);
    mainForm.setItemStateListener(this);
    display.setCurrent(mainForm);
  }
  
  public void startApp() {
  }
  
  public void pauseApp() {
     psend = null;    
  }
  
  public void destroyApp(boolean unconditional) {
     settings.save();    
  }
    
  private boolean checkSettings() {
      if(settings.atDefaults()) {
         Alert al = new Alert("Must Provide Settings!","In order to proceed further, you must go to the settings screen and provide your Username and PersonalKey.",null,AlertType.ERROR);
         al.setTimeout(Alert.FOREVER);
         display.setCurrent(al,mainForm);    
         return false;   // cannot proceed
      }
      return true; // ok to proceed
  }
  
  public void commandAction(Command c, Displayable s) {
    String cmdLabel = c.getLabel(); 
    if(cmdLabel.equals("Post")) {
       if(!checkSettings()) return;
       if(psend == null) psend = new PostSender(display);
       boolean success = psend.sendPost(settings.getUserName(),
                                        settings.getPersonalKey(),
                                        settings.getLocation(),
                                        post.getString(),
                                        PictureChoices.numbers[post_icon.getSelectedIndex()]);
       if(success) {
          post.setString(""); 
          post_icon.setSelectedIndex(0,true); // reset to "None"
          status_string.setText("ok");
       } else {
          // leave the user's post alone, and record the error.
          status_string.setText("Error while posting.");              
       } 
    }
    else if(cmdLabel.equals("View Overview")) {
       if(!checkSettings()) return;
       display.setCurrent(JaikuViewer.usersOverviewForm(settings.getUserName(),
                                                        settings.getPersonalKey(),
                                                        display,
                                                        mainForm));   
    }
    else if(cmdLabel.equals("View User")) {
       if(!checkSettings()) return;
       display.setCurrent(JaikuViewer.usersJaikusForm(settings.getUserName(),
                                                      display,
                                                      mainForm));   
    }
    else if(cmdLabel.equals("Settings")) {
       display.setCurrent(new SettingsForm(settings,mainForm,display));
    }
    else if(cmdLabel.equals("Exit")) {
       destroyApp(true);
       notifyDestroyed();
    }
    else if(cmdLabel.equals("About")) {
       display.setCurrent(new JaikuInfo(mainForm,display));    
    } 
 }
 
 public void itemStateChanged(Item item) {
    // if it's the post field, then update the number of characters remaining.
    if(item == post) {
       status_string.setText(Integer.toString(140 - post.size()));    
    }    
 }
}
