/*
 * JaikuViewer.java
 *
 *  Lovely Products 2007 (richard@movethemarkets.com)
 * GPL v2 or later.
 */

package com.lovely.bin;

import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;
import net.rim.device.api.system.*;
import java.util.*;
import org.json.me.*;

// *********************************************************
// *********************************************************
// *********************************************************
// *********************************************************
class GetUserContactsMenuItem extends MenuItem {
   private String userName;
   
   public GetUserContactsMenuItem() {
      super("_disabled_",2,2); 
      userName = null;
   }   
   
   public void setUserName(String un) {
     userName = un;   
     if(userName != null)  setText("Get " + un + "'s Contacts");
     else setText("_disabled_");
   }

   public void run() {
     if(userName != null) {
        ContactsViewer cv = ContactsViewer.contactsForm(userName);
        UiApplication.getUiApplication().pushScreen(cv); 
        cv.pullData();
     } 
   }
}

// *********************************************************
// *********************************************************
// *********************************************************
// *********************************************************
class ReloadMenuItem extends MenuItem {
   private String url;
   private boolean allowDrillDown;
   private boolean allowFetchComments;
   public ReloadMenuItem(String url_,boolean allowDrillDown_,boolean allowFetchComments_) {
      super("Reload",0,0);
      url = url_;
      allowDrillDown = allowDrillDown_;
      allowFetchComments = allowFetchComments_;
   }    
   
   public void run() {
      JaikuViewer jv = new JaikuViewer(url,allowDrillDown,allowFetchComments);
      UiApplication uiapp = UiApplication.getUiApplication(); 
      uiapp.popScreen(uiapp.getActiveScreen());  // get rid of this screen
      UiApplication.getUiApplication().pushScreen(jv);
      jv.pullData();
   }
}

// *********************************************************
// *********************************************************
// *********************************************************

class GetUserJaikuMenuItem extends MenuItem { 
  private String userName;
  public GetUserJaikuMenuItem() {
     super("_disabled_",1,1);
     userName = null;    
  }
  
  public void setUserName(String un) {
    userName = un;    
    if(userName != null) setText("Get " + un + "'s Jaikus");
    else setText("_disabled_");
  }
  
  public void run() {
     if(userName != null) {
       JaikuViewer jv = JaikuViewer.usersJaikusForm(userName);
       UiApplication.getUiApplication().pushScreen(jv);
       jv.pullData();         
     }
  }
}


// *********************************************************
// *********************************************************
// *********************************************************

class GetCommentsMenuItem extends MenuItem { 
  private String url;
  
  public GetCommentsMenuItem() {
     super("No Comments",1,1);
     url = null;    
  }
  
  public void setComments(String url_, int number) {
    if((url_ != null) && (number != 0)) {
      if(number == -1)   
         setText("Read All Comments");
      else 
         setText("Read " + Integer.toString(number) + " Comments");
      url =  url_;
   } else {
      setText("No Comments");    
      url = null;
   }
  }
  
  public void run() {
     if(url != null) {
       JaikuViewer jv = JaikuViewer.commentJaikusForm(url);
       UiApplication.getUiApplication().pushScreen(jv);
       jv.pullData();         
     }
  }
}


// *********************************************************
// *********************************************************
// *********************************************************
class AuthorTextField extends RichTextField {
   private String author;
   private boolean isComment;
   private int numComments;
   private String theURL;
   private String theID;
   public AuthorTextField(String author_,
                          String id_,
                          String theURL_,
                          boolean isComment_,
                          int numComments_,
                          String txt,
                          int[] offsets, 
                          byte[] attributes,
                          Font[] fonts, 
                          long style) {
      super(txt,offsets,attributes,fonts,style);
      author = author_;    
      theID = id_;
      isComment = isComment_;
      numComments = numComments_;
      theURL = sanitizeURL(theURL_,author,theID);
   }    
   
   private String sanitizeURL(String url,String author,String id) {
       int indx = url.lastIndexOf('#');
       if(indx != -1) {
          url = url.substring(0,indx);
       }

       // JAIKU BUG WORKAROUND!!!  Their URLs are fucked.  They can point to foreign sites.
       if((url.indexOf("jaiku.com/presence") == -1) && 
          (!url.startsWith("http://jaiku.com/channel/"))) {
         url = "http://" + author + ".jaiku.com/presence/" + id;
       }
       
       return url; 
   }
   
   public String getAuthor() { return author; }
   public String getURL() { return theURL; }
   public int getNumComments() { return numComments; }
   
}

// *********************************************************
// *********************************************************
// *********************************************************
class JSONDataReader extends Thread implements com.lovely.bin.IStatusable {
   private JaikuViewer jv;
   private String title;
   private Vector fields;
   public JSONDataReader(JaikuViewer jv_) {
       jv = jv_;
       title = "";
       fields = new Vector();
   }
     
      
   private String getUserInfo(JSONObject post) throws JSONException {
       JSONObject userObj = post.getJSONObject("user");
       String answer = userObj.getString("nick");
       if(answer.length() == 0) return null;
       return answer;  
    }
    
    private RichTextField parsePost(JSONObject post) throws JSONException {
       StringBuffer answer = new StringBuffer();
       String theAuthor = getUserInfo(post);
       String theURL = post.getString("url");
       String theID = null;
       int numComments = 0;
       boolean isComment = false;
       
       if(theAuthor != null) answer.append(theAuthor);
       else answer.append("(<jaiku feed error>)");
       
       answer.append(": ");
       if(post.has("id")) {          
          // it is a post...
          theID = post.getString("id");
          answer.append(post.getString("title"));
          String loc = post.getString("content");
          if(loc.length() > 0) {
             answer.append(" (").append(loc).append(")");    
          }
          numComments = post.getInt("comments");
       } else if(post.has("comment_id")) {
          // it is a comment...
          theID = post.getString("comment_id");
          isComment = true;
          numComments = -1;  // there must be at least 1, but we don't know how many!!    
          answer.append(post.getString("content"));
          String loc = post.getString("entry_title");
          if(loc.length() > 30) loc = loc.substring(0,30).concat("...");
          if(loc.length() > 0) {
             answer.append(" (Re: ").append(loc).append(")");    
          }
       }    

       answer.append(" (");
       answer.append(post.getString("created_at_relative"));
       answer.append(")");
       int[] offsets = new int[3];
       offsets[0] = 0;
       
       if(theAuthor != null) offsets[1] = theAuthor.length();
       else offsets[1] = 20;
       
       offsets[2] = answer.length();
       Font[] fonts = new Font[2];
       fonts[1] = Font.getDefault();
       fonts[0] = fonts[1].derive(Font.BOLD);
       byte[] attributes = new byte[2];
       attributes[0] = 0;
       attributes[1] = 1;
       RichTextField rtf = new AuthorTextField(theAuthor,
                                               theID,
                                               theURL,
                                               isComment,
                                               numComments,
                                               answer.toString(),
                                               offsets,
                                               attributes,
                                               fonts,
                                               Field.READONLY); 
       return rtf;
    }

    private RichTextField parseCommentPost(JSONObject post) throws JSONException {
       StringBuffer answer = new StringBuffer();
       String theAuthor = getUserInfo(post);
       int numComments = 0;
       boolean isComment = false;
       String theURL = post.getString("url");
       String theID = post.getString("id");
       if(theAuthor != null) answer.append(theAuthor);
       else answer.append("(<jaiku feed error>)");
       
       answer.append(": ");

       // it is a post...
       answer.append(post.getString("title"));
       String loc = post.getString("content");
       if(loc.length() > 0) {
          answer.append(" (").append(loc).append(")");    
       }

       answer.append(" (");
       answer.append(post.getString("created_at_relative"));
       answer.append(")");
       int[] offsets = new int[3];
       offsets[0] = 0;
       if(theAuthor != null) offsets[1] = theAuthor.length();
       else offsets[1] = 20;
       offsets[2] = answer.length();
       Font[] fonts = new Font[2];
       fonts[1] = Font.getDefault();
       fonts[0] = fonts[1].derive(Font.BOLD);
       byte[] attributes = new byte[2];
       attributes[0] = 0;
       attributes[1] = 1;
       RichTextField rtf = new AuthorTextField(theAuthor,
                                               theID,
                                               theURL,
                                               isComment,
                                               numComments,
                                               answer.toString(),
                                               offsets,
                                               attributes,
                                               fonts,
                                               Field.READONLY); 
       return rtf;
    }
    
    private RichTextField parseComment(JSONObject post) throws JSONException {
       StringBuffer answer = new StringBuffer();
       String theAuthor = getUserInfo(post);
       int numComments = 0;
       boolean isComment = true;
       String theURL = post.getString("url");
       String theID = post.getString("id");
       
       if(theAuthor != null) answer.append(theAuthor);
       else answer.append("(<jaiku feed error>)");
       answer.append(": ");

       // it is a comment...
       answer.append(post.getString("content"));

       answer.append(" (");
       answer.append(post.getString("created_at_relative"));
       answer.append(")");
       int[] offsets = new int[3];
       offsets[0] = 0;
       if(theAuthor != null) offsets[1] = theAuthor.length();
       else offsets[1] = 20;
       offsets[2] = answer.length();
       Font[] fonts = new Font[2];
       fonts[1] = Font.getDefault();
       fonts[0] = fonts[1].derive(Font.BOLD);
       byte[] attributes = new byte[2];
       attributes[0] = 0;
       attributes[1] = 1;
       RichTextField rtf = new AuthorTextField(theAuthor,
                                               theID,
                                               theURL,
                                               isComment,
                                               numComments,
                                               answer.toString(),
                                               offsets,
                                               attributes,
                                               fonts,
                                               Field.READONLY); 
       return rtf;
    }       
   
    public void setStatus(final String status) {
      UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                jv.setStatus(status);
            }
        });   
    }

    public void doAlert(final String alstr) {
     UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                Dialog.alert(alstr);
            }
        });   
   }

    private void setFields() {
      UiApplication.getUiApplication().invokeLater(new Runnable() {
            public void run() {
                jv.setFields(title,fields);
            }
        });   
    }
    
   public void run() {
        String jsonString = HTTPInterface.readURL(jv.getURL(),this);
        setStatus("Done reading... formatting page...");

        try {
           JSONObject mainObj = new JSONObject(jsonString);
           if(mainObj.has("stream")) {
               // it's a normal post feed...
              title = mainObj.getString("title");
              JSONArray stream = mainObj.getJSONArray("stream");
           
              for(int i = 0; i < stream.length(); ++i) {
                JSONObject post = stream.getJSONObject(i);
                RichTextField item = parsePost(post);             
                fields.addElement(item);
              } 
          }
          else {
              // it's a comment feed...
              title = "Comments on Post";
              RichTextField item = parseCommentPost(mainObj);
              fields.addElement(item);
              JSONArray stream = mainObj.getJSONArray("comments");

              if(stream.length() != 1) 
                fields.addElement(new RichTextField(Integer.toString(stream.length()) + " COMMENTS -->"));
              else 
                fields.addElement(new RichTextField("1 COMMENT -->"));

              for(int i = 0; i < stream.length(); ++i) {
                 JSONObject comment = stream.getJSONObject(i);
                 item = parseComment(comment);
                 fields.addElement(item);    
              }
          }
        } catch(Exception e) {
          title = "ERROR!";
          fields.removeAllElements();
          fields.addElement(new RichTextField(e.toString())); 
        }
        
        setFields();
   }       
}


// *********************************************************
// *********************************************************
// *********************************************************

// used to change the menu as various authors gain the focus... used
// to drill down to other user's jaikus.
class FocusMenuAdjuster implements FocusChangeListener {
   private GetUserJaikuMenuItem userCommand;
   private GetCommentsMenuItem commentCommand;
   private GetUserContactsMenuItem userContacts;

   public FocusMenuAdjuster(GetUserJaikuMenuItem userCommand_,
                            GetCommentsMenuItem commentCommand_,
                            GetUserContactsMenuItem userContacts_) {
      userCommand = userCommand_;
      commentCommand = commentCommand_;
      userContacts = userContacts_;
   }
   public void focusChanged(Field f, int typ) {
     if(typ == FocusChangeListener.FOCUS_GAINED) {
        AuthorTextField atf = (AuthorTextField)f;
        if(userCommand != null) userCommand.setUserName(atf.getAuthor());
        if(commentCommand != null) commentCommand.setComments(atf.getURL(),atf.getNumComments());
        if(userContacts != null) userContacts.setUserName(atf.getAuthor());
     }    
   }
}


// *********************************************************
// *********************************************************
// *********************************************************


/**
 *   Creates a screen from a JSON feed of jaiku content.
 */
public class JaikuViewer extends MainScreen {
    private String url;
    private boolean allowDrillDown; // should there be a context menu for browsing to jaikus?
    private boolean allowFetchComments; // should there be a menu item for going to comments?
    private GetUserJaikuMenuItem getUserJaikuMenuItem; // used when drilling down...
    private GetCommentsMenuItem getCommentsMenuItem; // used for getting jaiku post comments...
    private GetUserContactsMenuItem getUserContactsMenuItem; // to grab a user's contacts
    private LabelField statusField;
    
    public JaikuViewer(String url_,boolean allowDrillDown_, boolean allowFetchComments_) {
        super();
        url = url_;
        allowDrillDown = allowDrillDown_;
        allowFetchComments = allowFetchComments_;
        getUserJaikuMenuItem = null;
        getCommentsMenuItem = null;
        getUserContactsMenuItem = new GetUserContactsMenuItem();
        
        if(allowFetchComments) {
          getCommentsMenuItem = new GetCommentsMenuItem();
        }      
        if(allowDrillDown) {
           getUserJaikuMenuItem = new GetUserJaikuMenuItem();
        } 
   
        LabelField title = new LabelField
         ("JaikuBerry Loading Page...", LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(title);
        statusField = new LabelField("Making Connection...");
        add(statusField);
    }
    
    public String getURL() { return url; }
    
    public void pullData() {
       // spawn a thread to read in the URL...
       JSONDataReader jdr = new JSONDataReader(this);
       jdr.start();    
    }
    
    // make our menu...
    public void makeMenu(Menu m, int i) {
       m.add(new ReloadMenuItem(url,allowDrillDown,allowFetchComments));
       if(allowDrillDown) {
          m.add(getUserJaikuMenuItem);    
       }
       if(allowFetchComments) {
           m.add(getCommentsMenuItem);
       }
       m.add(getUserContactsMenuItem);
       super.makeMenu(m,i);
    }   
    
    // expected to be called from the event thread
    public void setStatus(String status) {
        statusField.setText(status);
    }
    
    // expected to be called from the event thread...
    public void setFields(String title, Vector fieldV) {
       setTitle(new LabelField(title));
       
       if(fieldV.size() == 0) {
          statusField.setText("No jaikus... this user's jaikus are probably private.");    
          return;
       }
       
       // first remove the status field...
       delete(statusField);
       
       // if we're allowing drilldown, set the focus listener so we
       // always have the right contextual menu in place..
       FocusChangeListener ffl = new FocusMenuAdjuster(getUserJaikuMenuItem,
                                                       getCommentsMenuItem,
                                                       getUserContactsMenuItem);           
      
       // now add in all the fields we have...
       for (Enumeration e = fieldV.elements() ; e.hasMoreElements() ;) {
          Field fld = (Field)e.nextElement();
          if(fld instanceof AuthorTextField) {
              fld.setFocusListener(ffl);
          }
          add(fld);
          add(new SeparatorField());
       }
    }
         
    protected boolean keyCharUnhandled(char key, int status, int time) {
      boolean retval = false;
      
      if(key == Characters.LATIN_SMALL_LETTER_T) {
         scroll(Manager.TOPMOST);    
      } else if(key == Characters.LATIN_SMALL_LETTER_B) {
         scroll(Manager.BOTTOMMOST);              
      } else if(key == Characters.LATIN_SMALL_LETTER_R) {
         new ReloadMenuItem(this.url,this.allowDrillDown,this.allowFetchComments).run();    
      } else if((key == Characters.LATIN_SMALL_LETTER_J) || 
                (key == Characters.ENTER)) {
         if(allowDrillDown) {
           Field f = getFieldWithFocus();
           if(f instanceof AuthorTextField) {
             AuthorTextField atf = (AuthorTextField)f;
             if(atf.getAuthor() != null) {
               JaikuViewer jv = JaikuViewer.usersJaikusForm(atf.getAuthor());
               UiApplication.getUiApplication().pushScreen(jv);
               jv.pullData();  
             }
           }
         }    
      } else if(key == Characters.LATIN_SMALL_LETTER_C) {
           Field f = getFieldWithFocus();
           if(f instanceof AuthorTextField) {
             AuthorTextField atf = (AuthorTextField)f;
             if((atf.getNumComments() != 0) &&
                (atf.getURL() != null)) {
               JaikuViewer jv = JaikuViewer.commentJaikusForm(atf.getURL());
               UiApplication.getUiApplication().pushScreen(jv);
               jv.pullData();  
             }
           }     
      }
      else retval = super.keyCharUnhandled(key,status,time);
  
      return retval;
    }    
         
    public static JaikuViewer usersOverviewForm(String user,String key) {
        StringBuffer url = new StringBuffer("http://");
        url.append(user).append(".jaiku.com/contacts/feed/json?user=")
           .append(user).append("&personal_key=").append(key);
       
        return new JaikuViewer(url.toString(),true,true);
    }
    
    public static JaikuViewer usersJaikusForm(String user) {
        StringBuffer url = new StringBuffer("http://");
        boolean allowDrillDown = false;
        if(user.charAt(0) != '#') {
          // it's not a channel
          url.append(user).append(".jaiku.com/feed/json");
        } else {
          // it's a channel
          allowDrillDown = true; // on channels, let people drill down.
          url.append("jaiku.com/channel/").append(user.substring(1)).append("/feed/json");
        }
        return new JaikuViewer(url.toString(),allowDrillDown,true);   
    }
    
    public static JaikuViewer exploreForm() {
        return new JaikuViewer("http://jaiku.com/feed/json",true,true);    
    }
    
    public static JaikuViewer commentJaikusForm(String url) {
       return new JaikuViewer(url + "/json",true,false);    
    }
   
}
